@@ -756,6 +756,8 @@ sequences = [
             comment='select terminal id'),
     seq_CSI('DECCRTST', 'q', intermediates=(Intermediate.MINUS,), flags=Flags.NOP,
             comment='CRT saver time'),
+    seq_CSI('XTERM_VERSION', 'q', pintro=(ParameterIntro.GT,),
+            comment='request xterm version report'),
     seq_CSI('DECSTBM', 'r',
             comment='set top and bottom margins'),
     seq_CSI('DECSKCV', 'r', intermediates=(Intermediate.SPACE,), flags=Flags.NOP,
@@ -822,7 +824,7 @@ sequences = [
             comment='redefine character display attribute association'),
     seq_CSI('DECRPDE', 'w', intermediates=(Intermediate.DQUOTE,), flags=Flags.NOP,
             comment='report displayed extent'),
-    seq_CSI('DECRQPSR', 'w', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
+    seq_CSI('DECRQPSR', 'w', intermediates=(Intermediate.CASH,),
             comment='request presentation state report'),
     seq_CSI('DECEFR', 'w', intermediates=(Intermediate.SQUOTE,), flags=Flags.NOP,
             comment='enable filter rectangle'),
@@ -924,14 +926,18 @@ sequences = [
             comment='xterm request termcap/terminfo'),
     seq_DCS('DECLBAN', 'r', flags=Flags.NOP,
             comment='load banner message'),
-    seq_DCS('DECRQTSR', 's', intermediates=(Intermediate.CASH,),
-            comment='request terminal state report'),
+    seq_DCS('DECTSR', 's', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
+            direction = Direction.TTH,
+            comment='terminal state report'),
     seq_DCS('XDGSYNC', 's', pintro=(ParameterIntro.EQUAL,), flags=Flags.NOP,
             comment='synchronous update'),
     seq_DCS('DECRSPS', 't', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
             comment='restore presentation state'),
     seq_DCS('DECAUPSS', 'u', intermediates=(Intermediate.BANG,), flags=Flags.NOP,
             comment='assign user preferred supplemental sets'),
+    seq_DCS('DECPSR', 'u', intermediates=(Intermediate.CASH,), flags=Flags.NOP,
+            direction = Direction.TTH,
+            comment='presentation state report'),
     seq_DCS('DECLANS', 'v', flags=Flags.NOP,
             comment='load answerback message'),
     seq_DCS('DECLBD', 'w', flags=Flags.NOP,
