@@ -214,7 +214,7 @@ typedef enum {
         VTE_ALIGN_END         = 2U,
 } VteAlign;
 
-/**
+/*
  * VteUuidFormat:
  * @VTE_UUID_FORMAT_SIMPLE: simple format
  * @VTE_UUID_FORMAT_BRACED: braced format
@@ -223,7 +223,7 @@ typedef enum {
  *
  * An enumeration that specifies the format of an UUID.
  *
- * Since: 0.76
+ * Since: 0.78
  */
 typedef enum /*< flags >*/ {
         VTE_UUID_FORMAT_SIMPLE = 1u << 0,
@@ -232,4 +232,78 @@ typedef enum /*< flags >*/ {
         VTE_UUID_FORMAT_ANY = 0x7u,
 } VteUuidFormat;
 
+/**
+ * VtePropertyFlags:
+ * @VTE_PROPERTY_FLAG_NONE: no flags, default
+ *
+ * A flags type.
+ *
+ * Since: 0.78
+ */
+typedef enum /*< flags >*/ {
+        VTE_PROPERTY_FLAG_NONE = 0,
+} VtePropertyFlags;
+
+/**
+ * VtePropertyType:
+ * @VTE_PROPERTY_VALUELESS: no value, use for signalling
+ * @VTE_PROPERTY_BOOL: a bool
+ * @VTE_PROPERTY_INT: a signed 64-bit integer
+ * @VTE_PROPERTY_UINT: an unsigned 64-bit integer
+ * @VTE_PROPERTY_DOUBLE: a finite double-precision floating point number
+ * @VTE_PROPERTY_RGB: a color
+ * @VTE_PROPERTY_RGBA: a color with alpha
+ * @VTE_PROPERTY_STRING: a string
+ * @VTE_PROPERTY_DATA: binary data
+ * @VTE_PROPERTY_UUID: a UUID
+ * @VTE_PROPERTY_URI: a URI
+ *
+ * An enumeration type describing types of properties.
+ *
+ * Since: 0.78
+ */
+typedef enum {
+        VTE_PROPERTY_INVALID = -1, /*< skip >*/
+        VTE_PROPERTY_VALUELESS = 0,
+        VTE_PROPERTY_BOOL,
+        VTE_PROPERTY_INT,
+        VTE_PROPERTY_UINT,
+        VTE_PROPERTY_DOUBLE,
+        VTE_PROPERTY_RGB,
+        VTE_PROPERTY_RGBA,
+        VTE_PROPERTY_STRING,
+        VTE_PROPERTY_DATA,
+        VTE_PROPERTY_UUID,
+        VTE_PROPERTY_URI,
+} VtePropertyType;
+
+/**
+ * VtePropertyId:
+ * @VTE_PROPERTY_ID_CURRENT_DIRECTORY_URI: the ID of the %VTE_TERMPROP_CURRENT_DIRECTORY_URI termprop
+ * @VTE_PROPERTY_ID_CURRENT_FILE_URI: the ID of the %VTE_TERMPROP_CURRENT_FILE_URI termprop
+ * @VTE_PROPERTY_ID_XTERM_TITLE: the ID of the %VTE_TERMPROP_XTERM_TITLE termprop
+ * @VTE_PROPERTY_ID_CONTAINER_NAME: the ID of the %VTE_TERMPROP_CONTAINER_NAME termprop
+ * @VTE_PROPERTY_ID_CONTAINER_RUNTIME: the ID of the %VTE_TERMPROP_CONTAINER_RUNTIME termprop
+ * @VTE_PROPERTY_ID_CONTAINER_UID: the ID of the %VTE_TERMPROP_CONTAINER_UID termprop
+ * @VTE_PROPERTY_ID_SHELL_PRECMD: the ID of the %VTE_TERMPROP_SHELL_PRECMD termprop
+ * @VTE_PROPERTY_ID_SHELL_PREEXEC: the ID of the %VTE_TERMPROP_SHELL_PREEXEC termprop
+ * @VTE_PROPERTY_ID_SHELL_POSTEXEC: the ID of the %VTE_TERMPROP_SHELL_POSTEXEC termprop
+ *
+ * An enum containing the IDs of the always-installed termprops.
+ *
+ * Since: 0.78
+ */
+typedef enum {
+        VTE_PROPERTY_ID_CURRENT_DIRECTORY_URI = 0,
+        VTE_PROPERTY_ID_CURRENT_FILE_URI,
+        VTE_PROPERTY_ID_XTERM_TITLE,
+        VTE_PROPERTY_ID_CONTAINER_NAME,
+        VTE_PROPERTY_ID_CONTAINER_RUNTIME,
+        VTE_PROPERTY_ID_CONTAINER_UID,
+        VTE_PROPERTY_ID_SHELL_PRECMD,
+        VTE_PROPERTY_ID_SHELL_PREEXEC,
+        VTE_PROPERTY_ID_SHELL_POSTEXEC,
+	_VTE_PROPERTY_ID_MAX = 0x7ffffff, /*< skip >*/
+} VtePropertyId;
+
 G_END_DECLS
