@@ -833,7 +833,8 @@ Widget::constructed() noexcept
         gtk_event_controller_set_name(controller.get(), "vte-motion-controller");
         gtk_widget_add_controller(m_widget, controller.release());
 
-        auto const scroll_flags = GtkEventControllerScrollFlags(GTK_EVENT_CONTROLLER_SCROLL_VERTICAL);
+        auto const scroll_flags = GtkEventControllerScrollFlags(GTK_EVENT_CONTROLLER_SCROLL_VERTICAL |
+                                                                GTK_EVENT_CONTROLLER_SCROLL_HORIZONTAL);
         controller = vte::glib::take_ref(gtk_event_controller_scroll_new(scroll_flags));
         g_signal_connect(controller.get(), "scroll-begin",
                          G_CALLBACK(scroll_begin_cb), this);
@@ -896,6 +897,8 @@ Widget::css_changed(GtkCssStyleChange* change)
 
         auto need_resize = padding_changed();
 
+        m_terminal->widget_style_updated();
+
         if (need_resize)
                 gtk_widget_queue_resize(gtk());
 }
@@ -1398,6 +1401,14 @@ Widget::im_set_cursor_location(cairo_rectangle_int_t const* rect) noexcept
         gtk_im_context_set_cursor_location(m_im_context.get(), rect);
 }
 
+void
+Widget::im_activate_osk() noexcept
+{
+#if VTE_GTK == 4
+        gtk_im_context_activate_osk(m_im_context.get(), nullptr);
+#endif
+}
+
 #if VTE_GTK == 3
 
 unsigned
@@ -1717,6 +1728,19 @@ Widget::notify_scroll_value_changed()
 #endif // VTE_GTK == 4
 }
 
+void
+Widget::notify_termprops_changed(int const* props,
+                                 int n_props) noexcept
+{
+        auto retval = gboolean{};
+        g_signal_emit(object(),
+                      signals[SIGNAL_TERMPROPS_CHANGED],
+                      0, /* detail */
+                      props,
+                      n_props,
+                      &retval);
+}
+
 #if VTE_GTK == 3
 
 std::optional<ScrollEvent>
